#!/usr/local/bin/itcl_wish -f

# Viewport demo
# ----------------------------------------------------------------------
# Assumes that the samples classes are in ../lib or that their location
# is communicated through the WIGWAMLIB environment variable.
# ----------------------------------------------------------------------

if {[catch "set env(WIGWAMLIB)"]} {
    set WIGWAMLIB ../lib
} else {
    set WIGWAMLIB [set env(WIGWAMLIB)]
}

lappend auto_path $WIGWAMLIB

proc Apply {} {
    global horiz vert

    set bars ""
    if {![string match *neither $horiz]} {
	lappend bars $horiz
    }
    if {![string match *neither $vert]} {
	lappend bars $vert
    }
    .v configure -scroll $bars
}

Scanvas .c -width 300 -height 200 -scrollregion {0 0 600 400} -relief ridge
for {set v 20} {$v <= 200} {incr v 20} {
    .c create line 0 $v 300 $v
    set vv [expr $v + 200]
    .c create line 300 $vv 600 $vv
}
for {set h 20} {$h < 300} {incr h 20} {
    .c create line $h 220 $h 400
    set hh [expr $h + 300]
    .c create line $hh 0 $hh 200
}
Viewport .v .c

message .t -aspect 300 -justify center -text "This demonstration involves an\
Scanvas inside a Viewport. In other words the canvas can be scrolled either\
by dragging with middle mouse button 2 held down (scan scrolling) or by\
using scrollbars.

Specify which scrollbars you want to be present through the\
radiobuttons below and then press the apply button to configure the new\
attachments."

frame .f
frame .l
frame .m
frame .r
pack .l .m .r -in .f -side left -padx 5

radiobutton .left -text left -variable horiz
radiobutton .top -text top -variable vert -anchor w
pack .left .top -in .l -fill x -pady 5
.left select

radiobutton .right -text right -variable horiz
radiobutton .bottom -text bottom -variable vert
pack .right .bottom -in .m -fill x -pady 5

radiobutton .hneither -text neither -variable horiz
radiobutton .vneither -text neither -variable vert
pack .hneither .vneither -in .r -fill x -pady 5

frame .b
button .q -text quit -command exit
button .a -text apply -command Apply
pack .q .a -in .b -side left -padx 5

pack .v -padx 5 -pady 5
pack .t .f .b -pady 5

Window . -title "Viewport demo"
